# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Luca Marchetti <ximilian@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gio, Gtk, GObject # pylint: disable=E0611
import logging
logger = logging.getLogger('dostuff_lib')

from . helpers import get_builder, show_uri, get_help_uri

# This class is meant to be subclassed by DoStuffWindow.  It provides
# common functions and some boilerplate.
class Window(Gtk.Window):
    __gtype_name__ = "Window"
    filter_project = []
    filter_context = []
    filter_keyword = []

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    def __init__(self):
        pass

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseDoStuffWindow object.
        """
        builder = get_builder('DoStuffWindow')
        new_object = builder.get_object("dostuff_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a DoStuffWindow object with it in order to finish
        initializing the start of the new DoStuffWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.PreferencesDialog = None # class
        self.preferences_dialog = None # instance
        self.AboutDialog = None # class
        self.SortWindow = None #class
        self.sort_window = None # instance
        self.FilterDialog = None #class
        self.filter_dialog = None # instance
        self.AddDialog = None #class
        self.add_dialog = None # instance

        self.settings = Gio.Settings("net.launchpad.dostuff")
        #self.settings.connect('changed', self.on_preferences_changed) #TODO

        # Optional Launchpad integration
        # This shouldn't crash if not found as it is simply used for bug reporting.
        # See https://wiki.ubuntu.com/UbuntuDevelopment/Internationalisation/Coding
        # for more information about Launchpad integration.
        try:
            from gi.repository import LaunchpadIntegration # pylint: disable=E0611
            LaunchpadIntegration.add_items(self.ui.helpMenu, 1, True, True)
            LaunchpadIntegration.set_sourcepackagename('dostuff')
        except ImportError:
            pass

        # Optional application indicator support
        # Run 'quickly add indicator' to get started.
        # More information:
        #  http://owaislone.org/quickly-add-indicator/
        #  https://wiki.ubuntu.com/DesktopExperienceTeam/ApplicationIndicators
        try:
            from dostuff import indicator
            # self is passed so methods of this class can be called from indicator.py
            # Comment this next line out to disable appindicator
            self.indicator = indicator.new_application_indicator(self)
        except ImportError:
            pass

    def on_mnu_contents_activate(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri())

    def on_mnu_about_activate(self, widget, data=None):
        """Display the about box for dostuff."""
        if self.AboutDialog is not None:
            about = self.AboutDialog() # pylint: disable=E1102
            response = about.run()
            about.hide()

    def on_mnu_preferences_activate(self, widget, data=None):
        """Display the preferences window for dostuff."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Preferences from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        self.check_and_create_preferences_dialog()
        self.preferences_dialog.present()
        # destroy command moved into dialog to allow for a help button

    def check_and_create_preferences_dialog(self):
        if self.preferences_dialog is None:
            if self.PreferencesDialog is not None:
                logger.debug('create new preferences_dialog')
                self.preferences_dialog = self.PreferencesDialog() # pylint: disable=E1102
                self.preferences_dialog.connect('destroy', self.on_preferences_dialog_destroyed)

    def on_mnu_sort_activate(self, widget, data=None):
        """Display the sort window for dostuff."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Sort from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        self.check_and_create_sort_window()
        self.sort_window.present()
        # destroy command moved into dialog to allow for a help button

    def check_and_create_sort_window(self):
        if self.sort_window is None:
            if self.SortWindow is not None:
                logger.debug('create new sort_window')
                self.sort_window = self.SortWindow() # pylint: disable=E1102
                self.sort_window.connect('destroy', self.on_sort_window_destroyed)
                
    def on_mnu_filter_activate(self, widget, data=None):
        """Display the filter window for dostuff."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Filter from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        self.check_and_create_filter_dialog()
        self.filter_dialog.present()
        # destroy command moved into dialog to allow for a help button

    def check_and_create_filter_dialog(self):
        if self.filter_dialog is None:
            if self.FilterDialog is not None:
                logger.debug('create new filter_dialog')
                self.filter_dialog = self.FilterDialog() # pylint: disable=E1102
                self.filter_dialog.connect('destroy', self.on_filter_dialog_destroyed)
                for p in self.filter_project:
                    self.filter_dialog.add_filter_project(p)
                for c in self.filter_context:
                    self.filter_dialog.add_filter_context(c)
                for k in self.filter_keyword:
                    self.filter_dialog.add_filter_keyword(k)
                self.filter_dialog.update_choices()
                self.filter_dialog.reset()
                self.filter_dialog.connect("filter_signal", self.report_filter_signal)

    def on_mnu_add_activate(self, widget, data=None):
        """Display the add window for dostuff."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Add from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        self.check_and_create_add_dialog()
        self.add_dialog.update_choices(self.filter_dialog.filter_catalog)
        self.add_dialog.present()
        # destroy command moved into dialog to allow for a help button

    def check_and_create_add_dialog(self):
        if self.add_dialog is None:
            if self.AddDialog is not None:
                logger.debug('create new add_dialog')
                self.add_dialog = self.AddDialog() # pylint: disable=E1102
                self.add_dialog.connect('destroy', self.on_add_dialog_destroyed)
                self.check_and_create_filter_dialog()
                self.add_dialog.update_choices(self.filter_dialog.filter_catalog)
                self.add_dialog.connect("add_signal", self.report_add_signal)
        self.add_dialog.reset()

    def on_mnu_close_activate(self, widget, data=None):
        """Signal handler for closing the DoStuffWindow."""
        self.destroy()

    def on_mnu_new_activate(self, widget, data=None):
        pass

    def on_mnu_open_activate(self, widget, data=None):
        pass

    def on_destroy(self, widget, data=None):
        """Called when the DoStuffWindow is closed."""
        # Clean up code for saving application state should be added here.
        Gtk.main_quit()

    def on_preferences_changed(self, settings, key, data=None):
        pass
        print 'preference changed: %s = %s' % (key, str(settings.get_value(key)))


    def on_preferences_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the preferences dialog'''
        logger.debug('on_preferences_dialog_destroyed')
        # to determine whether to create or present preferences_dialog
        self.preferences_dialog = None

    def on_sort_window_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the sort window'''
        logger.debug('on_sort_window_destroyed')
        # to determine whether to create or present sort_window
        self.sort_window = None

    def on_filter_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the filter dialog'''
        logger.debug('on_filter_dialog_destroyed')
        # to determine whether to create or present filter_dialog
        self.filter_dialog = None

    def on_add_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the add dialog'''
        logger.debug('on_add_dialog_destroyed')
        # to determine whether to create or present add_dialog
        self.add_dialog = None

GObject.type_register(Window)
GObject.signal_new("edit_signal", Window, GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE, (GObject.TYPE_STRING, GObject.TYPE_STRING, GObject.TYPE_PYOBJECT,))
